<?php

require_once( 'NikkiObject.class.php' );
require_once( 'MDB2.php' );

// データベースの条件
// テーブル名は小文字であること
// フィールド名id（integer auto_increment primary key）を持つこと
// すべてのフィールドはデフォルト値を持つこと

class DBTable extends NikkiObject {
	
	var $mdb2;				// MDB2オブジェクト
	var $table;				// テーブル名
	var $id;				// 関連づけられているレコードID
	var $is_update;			// 変更があったらtrue
	var $do_transaction;	// トランザクションの利用の可否
	
	// テーブル構造
	var $definition = array(
		'id' => array( 
			'type' => 'integer',
			'autoincrement' => true,
		)
	);
	// テーブルオプション（UTF-8を使う場合）
	var $table_options = array(
		'charset' => 'utf8',
		'collate' => 'utf8_unicode_ci',
	 );
	
	// オブジェクト作成メソッド
    function DBTable( $dsn , $property = null, $value = null ) {
		// 親コンストラクタの呼び出し
		parent::NikkiObject();
		
		// クラス名をデータベーステーブル名にする
		$this->table = get_class( $this );
		// $this->mdb2 =& MDB2::singleton( $dsn ,array( 'default_table_type'=>'INNODB', 'use_transactions' => true ) );
		
		$this->mdb2 =& MDB2::singleton( $dsn );
		if( MDB2::isError( $this->mdb2 ) ) {
			$this->setError( $this->mdb2->getMessage() );
			$this->id = 0;
			return;
		}
		// トランザクションのサポート
		$this->do_transaction = false;
		if( $this->do_transaction ) {
			$this->mdb2->beginTransaction();
		}
		
		// 無効レコードを作成する
		if( ($property == null) || ($value == null) ) {
			$this->id = 0;
		}
		// 空のレコードを新規作成
		else if( ($property == 'id') && ($value == 0) ) {
			$sqlstr = "INSERT INTO ".$this->table." VALUES ( )";
			$res = $this->mdb2->exec( $sqlstr );
			if( MDB2::isError( $res ) ) {
				if( $this->do_transaction ) $this->mdb2->rollback();
				$this->setError( $res->getMessage() );
				$this->mdb2->disconnect();
				$this->id = 1;
				return;
			}
			$this->id = $this->mdb2->lastInsertID();
		}
		// 渡されたpropertyのレコードを検索
		else {
			$this->mdb2->setFetchMode(MDB2_FETCHMODE_ASSOC);
			$sqlstr = "SELECT * FROM ".$this->table.
			            " WHERE ".$property.
			              "=".$this->mdb2->quote( $value );
			
			$res = $this->mdb2->query( $sqlstr );
			if( MDB2::isError( $res ) ) {
				if( $this->do_transaction ) $this->mdb2->rollback();
				$this->setError( $res->getMessage() );
				$this->mdb2->disconnect();
				$this->id = 0;
				return;
			}
			$row = $res->fetchOne();
			if( MDB2::isError( $row ) || $row === false ) {
				if( $this->do_transaction ) $this->mdb2->rollback();
				$this->setError( $row->getMessage() );
				$this->mdb2->disconnect();
				$this->id = 0;
				return;
			}
			$this->id = $row;
		}
		// DBTableオブジェクト作成成功
		$this->is_update = false;
		return;
	}
	
	// テーブルの作成
	function create() {
		// PHP4ではManagerモジュールのロードを行う必要がある
		// PHP5では不要
		$this->mdb2->loadModule('Manager');
		// PHP4ではMDB2オブジェクトのmnagerのメソッドを呼ぶ必要がある
		// PHP5ではMDB2のメソッドとして呼び出すことが可能
		$ret = $this->mdb2->manager->createTable( $this->table, $this->definition ,$this->table_options );
		if( MDB2::isError( $ret ) ) {
			$this->setError( $ret->getMessage() );
		}
		return;
	}
	
	// テーブル構造配列を得る
	function getDefinition() {
		return $this->definition;
	}
	
	// スラッシュを取る
	function strips( & $value ) {
		if( is_array($value) ) {
			$value = array_map( array("DBTable", "strips"), $value );
		}
		else {
			$value = stripslashes( $value );
		}
		return $value;
	}
	
	// SELECT文発行（汎用）
	function fetch_array( $option = null, $arr = array() ) {
		$retval = array();
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		$this->mdb2->setFetchMode(MDB2_FETCHMODE_ASSOC);
		$sqlstr = "SELECT * FROM ".$this->table;
		if( $option != null ) {
			$sqlstr .= " " . $option;
		}
		$sth = $this->mdb2->prepare( $sqlstr , null, MDB2_PREPARE_RESULT );
		$res = $sth->execute($arr);
		if( MDB2::isError( $res ) ) {
			$this->setError( $res->getMessage() );
			return false;
		}
		while ($row = $res->fetchRow() ){
			array_push( $retval, $row );
		}
		DBTable::strips($retval);
		return $retval;
	}
	
	// UPDATE文発行（汎用）
	function update( $option, $arr ) {
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		$sqlstr = "UPDATE ".$this->table." SET ".$option;
		$sth = $this->mdb2->prepare( $sqlstr , null, MDB2_PREPARE_MANIP );
		$res = $sth->execute( $arr );
		if(MDB2::isError($res) ){
			$this->setError( $res->getMessage() );
			return;
		}
		$this->is_update = true;
		return;
	}
	
	// DELETE文発行（汎用）
	function delete( $option, $arr ) {
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		$sqlstr = "DELETE FROM ".$this->table." ".$option;
		$sth = $this->mdb2->prepare( $sqlstr , null, MDB2_PREPARE_MANIP );
		$res = $sth->execute( $arr );
		if(MDB2::isError($res) ){
			$this->setError( $res->getMessage() );
			return;
		}
		$this->is_update = true;
		return;
	}
	
	// 値の代入
	function set( $property, $value ) {
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		if( $this->id <= 0 ) {
			$this->setError( "0のidには代入はできません" );
			return false;
		}
		if( $property == "id" ) {
			$this->setError( "idの変更はできません" );
			return false;
		}
		$sqlstr = "UPDATE ".$this->table." SET ".$property."=".
		             $this->mdb2->quote($value)." WHERE id='".$this->id."'";
		$res = $this->mdb2->exec( $sqlstr );
		if( MDB2::isError( $res) ) {
			$this->setError( $res->getMessage() );
			return;
		}
		$this->is_update = true;
		return true;
	}
	
	// 値の取得
	function get( $property ) {
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		if( $this->id <= 0 ) {
			$this->setError( "0のidの値は取得できません" );
			return false;
		}
		if( $property == "id" ) return $this->id;
		$this->mdb2->setFetchMode(MDB2_FETCHMODE_ASSOC);
		$sqlstr = "SELECT ".$property." FROM ".$this->table." WHERE id='".$this->id."'";
		$res = $this->mdb2->query($sqlstr);
		if( MDB2::isError( $res) ) {
			$this->setError( $res->getMessage() );
			return;
		}
		$value = $res->fetchOne();
		if( $value === false || MDB2::isError( $value ) ) {
			$this->setError( $property."は存在しません" );
			return;
		}
		return stripslashes($value);
	}
	
	// このレコードを削除する
	function remove() {
		if( $this->mdb2 == null ) {
			$this->setError( "このオブジェクトは無効です" );
			return false;
		}
		if( $this->id <= 0 ) {
			$this->setError( "0のidは削除できません" );
			return false;
		}
		$option = "WHERE id=?";
		$arr = array( $this->id );
		$ret = $this->delete( $option, $arr );
		if( MDB2::isError($ret) ) {
			$this->setError( $res->getMessage() );
			return false;
		}
		$this->id = 0;
		return $ret;
	}
	
	// 終了処理
	function close() {
		if( $this->do_transaction ) {
			if( $this->is_update ) $this->mdb2->commit();
			else $this->mdb2->rollback();
		}
		$this->mdb2->disconnect();
		$this->id = 0;
		if(! MDB2::isConnection($this->mdb2) ) {
			$this->mdb2 = null;
		}
	}
}
?>