<?php

require_once( 'DBTable.class.php' );

// アクセス許可フラグ

define( 'TBL_WRITE_PERMIT_MASK'  , 0x0F );
define( 'TBL_ADMIN_WRITE_PERMIT' , 0x04 );
define( 'TBL_USER_WRITE_PERMIT'  , 0x02 );
define( 'TBL_PUB_WRITE_PERMIT'   , 0x01 );
define( 'TBL_READ_PERMIT_MASK'   , 0xF0 );
define( 'TBL_ADMIN_READ_PERMIT'  , 0x40 );
define( 'TBL_USER_READ_PERMIT'   , 0x20 );
define( 'TBL_PUB_READ_PERMIT'    , 0x10 );

define( 'PUB'    , 0x11 );
define( 'USER'   , 0x33 );
define( 'ADMIN'  , 0xFF );

// 日記テーブルクラス
// 現ユーザーのアクセス権は
// $_SESSION['permission']に格納されている
//
class NikkiTable extends DBTable {
	
	// デフォルトのアクセス権は管理者のみ
	var $permission = 0x44;
	
	// コンストラクタ
	function NikkiTable( $dsn, $property = NULL , $value = NULL) {
		global $_SESSION;
		
		// 現在のユーザーの権限が設定されていなければ
		// 無条件に一般読み出し権を与える
		if( ! isset( $_SESSION['permission'] ) )
			$_SESSION['permission'] = TBL_PUB_READ_PERMIT;
		parent::DBTable( $dsn, $property, $value );
	}
	
	// このテーブルの許可証を得る
	function getPermission() {
		return $this->permission;
	}
	
	// 権限チェック
	function checkWritePermit() {
		global $_SESSION;
		return ($_SESSION['permission'] & $this->permission & TBL_WRITE_PERMIT_MASK);
	}

	function checkReadPermit() {
		global $_SESSION;
		return ($_SESSION['permission'] & $this->permission & TBL_READ_PERMIT_MASK);
	}
	
	// $valueの無害化メソッド
	function sanitate( $value ) {
		if( is_array( $value ) ) {
			return array_map( array("NikkiTable", "sanitate"), $value );
		}
		else {
			return htmlspecialchars( $value, ENT_QUOTES );
		}
	}
	
	// SELECT文発行
	function fetch_array($option = null, $arr = array() ) {
		if( $this->checkReadPermit() ) {
			$ret = parent::fetch_array($option, $arr);
			
			// 一般書き込み可なら読み出し時に無害化を強制
			if( $this->permission & TBL_PUB_WRITE_PERMIT )
				$ret = $this->sanitate( $ret );
			
			return $ret;
		}
		else {
			$this->setError("権限がありません");
			return null;
		}
	}
	
	// UPDATE文発行
	function update( $option, $arr ) {
		if( $this->checkWritePermit() ) {
			return parent::update($option, $arr);
		}
		else {
			$this->setError("権限がありません");
			return false;
		}
	}
	
	// DELETE文発行（汎用）
	function delete( $option, $arr ) {
		if( $this->checkWritePermit() ) {
			return parent::delete($option, $arr);
		}
		else {
			$this->setError("権限がありません");
			return false;
		}
	}
	
	// 値の代入
	function set( $property, $value ) {
		if( $this->checkWritePermit() ) {
			return parent::set($property, $value);
		}
		else {
			$this->setError("権限がありません");
			return false;
		}
	}
	
	// 値の取得
	function get( $property ) {
		if( $this->checkReadPermit() ) {
			$ret = parent::get($property);
			// 一般書き込み可なら読み出し時に無害化を強制
			if( $this->permission & TBL_PUB_WRITE_PERMIT )
				$ret = NikkiTable::sanitate( $ret );
			return $ret;
		}
		else {
			$this->setError("権限がありません");
			return false;
		}
	}
	
	// このレコードを削除する
	function remove() {
		if( $this->checkWritePermit() ) {
			return parent::remove();
		}
		else {
			$this->setError("権限がありません");
			return false;
		}
	}
}
?>